package ACL2::Meta;

=head1 NAME

ACL2::Meta - fE<oacute>rmula a demostrar dentro de una demostraciE<oacute>n

=cut

#
# Módulo de procesado de demostraciones anidadas
# Copyright (C) 2008 Antonio García Domínguez
# $Id$
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

use strict;
use warnings;
use ACL2::Localizacion;
use ACL2::Proceso;
use ACL2::Proceso::Fabrica;
use ACL2::Proceso::AbandonarMetaPrincipal;
use ACL2::Proceso::Contradiccion;
use ACL2::Proceso::Descartar;
use ACL2::Proceso::ElimDestructores;
use ACL2::Proceso::Fracaso;
use ACL2::Proceso::Generalizacion;
use ACL2::Proceso::Induccion;
use ACL2::Proceso::PreInduccion;
use ACL2::Proceso::Simplificacion;
use ACL2::Proceso::UsoHipotesis;

my $lh = ACL2::Localizacion->get_handle();

=head1 FUNCTIONS

Las funciones que aporta este mE<oacute>dulo son:

=head2 new

Uso: Constructor.

Responsabilidad: Crea una nueva meta, sin lanzar su filtrado.

ParE<aacute>metros: 

=over 2

=item *

etiq - cadena con la etiqueta completa (ej.: Subgoal *1/2'3')

=item *

salida - cadena con la salida completa (etiqueta+fE<oacute>rmula+resto)

=back

=cut

sub new {
    my ($clase, $etiq, $salida) = @_;

    my $ref = bless {
	"etiq" => $etiq,
	"salida" => $salida,
	"rv_submetas" => [],
    }, $clase;

    $ref->_inicializar();

    return $ref;
}


=head2 agregar_submeta

Uso: ME<eacute>todo pE<uacute>blico.

Responsabilidad: AE<ntilde>ade una ACL2::Meta a la lista de submetas.

ParE<aacute>metros: submeta - ACL2::Meta a agregar.

Devuelve: nada.

=cut

sub agregar_submeta
{
    my ($this, $submeta) = @_;
    push @{$this->{rv_submetas}}, $submeta;
}


=head2 escribir_xml

Uso: ME<eacute>todo pE<uacute>blico.

Responsabilidad: Escribe la informaciE<oacute>n de esta meta y sus
submetas en formato XML.

ParE<aacute>metros: writer - objeto XML::Writer destino de la salida

Devuelve: nada.

=cut

sub escribir_xml {
    my ($this, $writer) = @_;

    # Tag inicial
    $writer->startTag(ACL2::Tags::SUBGOAL_TAG, 
		      "label" => $this->{etiq}, 
		      "formula" => $this->{formula});

    # Proceso seguido - inicio
    $this->{proceso}->escribir_xml_cabecera($writer);

    # Submetas
    foreach my $submeta (@{$this->{rv_submetas}}) {
	$submeta->escribir_xml($writer);
    }	

    # Proceso seguido - final
    $this->{proceso}->escribir_xml_final($writer);

    # Tag final
    $writer->endTag(ACL2::Tags::SUBGOAL_TAG);
}


=head2 _inicializar

Uso: ME<eacute>todo privado.

Responsabilidad: Lee la informaciE<oacute>n contenida en la salida de
la meta. No se ocupa de las submetas.

ParE<aacute>metros: Ninguno.

Devuelve: nada.

=cut

sub _inicializar
{
    caller eq __PACKAGE__ or die;
    my ($this) = @_;

    # Obtenemos las partes de la meta
    ($this->{formula}, $this->{texto}) = $this->{salida} =~ /^\s*(.*?)\.\s*(.*)/os 
	or die $lh->maketext("No se pudieron obtener las partes de la meta.");
    
    $this->{proceso} = ACL2::Proceso::Fabrica->crear($this->{texto});
    $this->{proceso}->isa('ACL2::Proceso') 
	or die $lh->maketext("Se esperaba un objeto de la clase [_1]", "ACL2::Proceso");
}

1;

